open Ast

(* Example source program expressions --------------------------------------- *)

(* x1 + (x2 + (x3 + (x4  (x5 + (x6 + (x6 + (x7 + x8))))))) *)
let sum : Ast.exp =
  (Add(Var "x1",
       Add(Var "x2",
           Add(Var "x3",
               Add(Var "x4",
                   Add(Var "x5",
                       Add(Var "x6",
                           Add(Var "x7",
                               Var "x8"))))))))
 

(* x1 * (x2 * (x3 * (x4  (x5 * (x6 * (x6 * (x7 * x8))))))) *)
let product : Ast.exp =
  (Mul(Var "x1",
       Mul(Var "x2",
           Mul(Var "x3",
               Mul(Var "x4",
                   Mul(Var "x5",
                       Mul(Var "x6",
                           Mul(Var "x7",
                               Var "x8"))))))))


(* x1 * x1 *)
let square : Ast.exp =
  (Mul(Var "x1", Var "x1"))


(* (2 * (x1 + x2)) *)
let perimeter : Ast.exp = Mul (Const 2L, Add (Var "x1", Var "x2"))

let sum2 : Ast.cmd =
  Seq (Assn ("t", Var "x1"),
       Seq (Assn ("t", Add (Var "t", Var "x2")),
            Seq (Assn ("t", Add (Var "t", Var "x3")),
                 Seq (Assn ("t", Add (Var "t", Var "x4")),
                      Seq (Assn ("t", Add (Var "t", Var "x5")),
                           Seq (Assn ("t", Add (Var "t", Var "x6")),
                                Seq (Assn ("t", Add (Var "t", Var "x7")),
                                     Seq (Assn ("t", Add (Var "t", Var "x8")),
                                          Return (Var "t")))))))))
                                          
let early_exit : Ast.cmd =
  Seq (Assn ("t", Var "x1"),
       Seq (Assn ("t", Add (Var "t", Var "x2")),
            Seq (Return (Var "t"),
                 Assn ("t", Mul (Var "t", Const 2L)))))

let abs : Ast.cmd =
  IfLZ (Var "x1", Return (Neg (Var "x1")), Return (Var "x1"))

let factorial : Ast.cmd =
  Seq (Seq (Assn ("result", Const 1L),
            Assn ("i", Const 0L)),
       Seq (WhileLZ (Add (Var "i", Neg (Var "x1")),
                     Seq (Assn ("i", Add (Var "i", Const 1L)),
                          Assn ("result", Mul (Var "result", Var "i")))),
            Return (Var "result")))

(* compilation -------------------------------------------------------------- *)

(*
let () =
  let open Compile in
  let c = IR.mk_context () in
  print_endline (IR.pp_program (translate_exp c sum))
*)

(* Resulting X86 program after compilation *)
let tgt : X86.prog = Compile3.compile factorial

(* Output the resulting s file *)
let s = X86.string_of_prog tgt

;; print_endline s
